import { SET_PRODUCTS, SET_LOADING_PRODUCTS, SET_PRODUCT_WITH_VARIANT } from '@/constants/mutationTypes'
import { INIT_PRODUCTS, SELECT_PRODUCT, ADD_TO_CART } from '@/constants/actionTypes'

const initialState = {
    products: [],
    loadingProducts: false,
    showVariants: false,
    productWithVariant: null
}

const state = {
    products: initialState.products,
    loadingProducts: !initialState.loadingProducts,
    showVariants: initialState.showVariants,
    productWithVariant: initialState.productWithVariant
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    products(state) {
        return state.products
    },
    /**
     * 
     * @param {*} state 
     */
    loadingProducts(state) {
        return state.loadingProducts
    },
    /**
     * 
     * @param {*} state 
     */
    showVariants(state) {
        return state.showVariants
    },
    /**
     * 
     * @param {*} state 
     */
    productWithVariant(state) {
        return state.productWithVariant
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_PRODUCTS] (state, payload) {
        state.products = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_LOADING_PRODUCTS] (state, payload) {
        state.loadingProducts = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    [SET_PRODUCT_WITH_VARIANT] (state, payload) {
        state.productWithVariant = payload
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [INIT_PRODUCTS] ({ commit }, payload) {
        commit(SET_PRODUCTS, payload)
        commit(SET_LOADING_PRODUCTS)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    [SELECT_PRODUCT] ({ commit, dispatch }, payload) {
        if (payload.variantCount > 1) {
            commit(SET_PRODUCT_WITH_VARIANT, payload)
            return
        }

        dispatch(ADD_TO_CART, payload.variants[0])
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}

